@@ -13,34 +13,49 @@ use Bio::Grid::Run::SGE::Log::Worker;
 use Bio::Grid::Run::SGE::Log::Notify::Jabber;
 use Bio::Grid::Run::SGE::Log::Notify::Mail;
 use Bio::Grid::Run::SGE::Config;
+use Bio::Gonzales::Util::Cerial;
+
 use Sys::Hostname;
 
 our $VERSION = 0.01_01;
 
-has config_file    => ( is => 'rw', required   => 1 );
-has c              => ( is => 'rw', required   => 1 );
-has error_cmd_file => ( is => 'rw', lazy_build => 1 );
-has error_log_file => ( is => 'rw', lazy_build => 1 );
-has attempts       => ( is => 'rw', lazy_build => 1 );
-has _log_report    => ( is => 'rw', default    => sub { [] } );
-has _cmd_script    => ( is => 'rw', default    => sub { [] } );
+has config_file         => ( is => 'rw', required   => 1 );
+has c                   => ( is => 'rw', required   => 1 );
+has failed_restart_file => ( is => 'rw', lazy_build => 1 );
+has failed_log_file     => ( is => 'rw', lazy_build => 1 );
+has failed_update_file  => ( is => 'rw', lazy_build => 1 );
+has attempts            => ( is => 'rw', lazy_build => 1 );
+has _log                => ( is => 'rw', default    => sub { [] } );
+has _node_log           => ( is => 'rw', default    => sub { [] } );
+has _cmd_script         => ( is => 'rw', default    => sub { [] } );
+has num_jobs            => ( is => 'rw', default    => 0 );
+has num_jobs_skipped            => ( is => 'rw', default    => 0 );
+has num_jobs_failed     => ( is => 'rw', default    => 0 );
 
-sub _build_error_log_file {
+sub _build_failed_update_file {
   my ($self) = @_;
 
   my ( $tmp_dir, $job_name, $job_id ) = @{ $self->c }{qw/tmp_dir job_name job_id/};
-  my $error_log_file = catfile( $tmp_dir, "log.$job_name.j$job_id.txt" );
-
-  return $error_log_file;
+  my $update_log_file = catfile( $tmp_dir, "update.$job_name.j$job_id.sh" );
+  return $update_log_file;
 }
 
-sub _build_error_cmd_file {
+sub _build_failed_log_file {
   my ($self) = @_;
 
   my ( $tmp_dir, $job_name, $job_id ) = @{ $self->c }{qw/tmp_dir job_name job_id/};
-  my $error_cmd_file = catfile( $tmp_dir, "restart.$job_name.j$job_id.sh" );
+  my $failed_log_file = catfile( $tmp_dir, "log.$job_name.j$job_id.txt" );
 
-  return $error_cmd_file;
+  return $failed_log_file;
+}
+
+sub _build_failed_restart_file {
+  my ($self) = @_;
+
+  my ( $tmp_dir, $job_name, $job_id ) = @{ $self->c }{qw/tmp_dir job_name job_id/};
+  my $failed_restart_file = catfile( $tmp_dir, "restart.$job_name.j$job_id.sh" );
+
+  return $failed_restart_file;
 }
 
 sub _build_attempts {
@@ -50,7 +65,13 @@ sub _build_attempts {
 
 sub _report_log {
   my $self = shift;
-  push @{ $self->_log_report }, @_;
+  push @{ $self->_log }, @_;
+  return;
+}
+
+sub _report_node_log {
+  my $self = shift;
+  push @{ $self->_node_log }, @_;
   return;
 }
 
@@ -73,9 +94,12 @@ sub analyse {
   my $log_dir = my_glob( $c->{log_dir} );
 
   my $something_crashed;
+  my $num_skipped = 0;
 
-  $self->_report_log( $self->_gen_subject(),                     '' );
-  $self->_report_log( 'main log file: ' . $self->error_log_file, '' );
+  $self->_report_log( 'main log file: ' . $self->failed_log_file, '' );
+
+  $self->_report_log( 'restart failed jobs: ' . $self->failed_restart_file );
+  $self->_report_log( 'update job states: ' . $self->failed_update_file, '' );
 
   $self->_report_log( 'working dir: ' . $c->{working_dir} );
   $self->_report_log( 'result dir: ' . $c->{result_dir} );
@@ -147,13 +171,15 @@ sub analyse {
       );
       # track which nodes broke, often one specific node always breaks
       $err_hosts{ $log_data->{hostname} }++;
+    } elsif ( exists( $log_data->{'comp.task.exit.skip'} ) ) {
+      $num_skipped++;
     }
   }
 
   my $no_jobs_ran_at_all;
-  my $total_jobs = 'n/a';
+  my $num_jobs = 'n/a';
   if ( exists( $c->{range} ) ) {
-    $total_jobs = $c->{range}[1] - $c->{range}[0] + 1;
+    $num_jobs = $c->{range}[1] - $c->{range}[0] + 1;
   MISSING_JOBS:
     for ( my $i = $c->{range}[0]; $i <= $c->{range}[1]; $i++ ) {
       unless ( exists( $jobs_with_log{$i} ) ) {
@@ -170,29 +196,38 @@ sub analyse {
           $err_hosts{NA}++;
         } else {
           $no_jobs_ran_at_all = 1;
-          $err_hosts{NA} = $total_jobs;
+          $err_hosts{NA} = $num_jobs;
           last MISSING_JOBS;
         }
       }
     }
   }
 
-  $self->_report_log("obviously, no jobs were run at all") if ($no_jobs_ran_at_all);
-  if ($something_crashed) {
+  $self->num_jobs($num_jobs);
+  $self->num_jobs_skipped($num_skipped);
+
+  $self->_report_log(($num_jobs || 'n/a') . " jobs in total");
+  $self->_report_log('(' .$num_skipped . " jobs did not need to run again)") if($num_skipped);
+  if ($no_jobs_ran_at_all) {
+    $self->_report_log("obviously, no jobs were run at all");
+    $self->num_jobs_failed($num_jobs);
+  } elsif ($something_crashed) {
     # create a log entry with the hostnames and the number of times crashed.
     # often one node has all crashes and with this you can spot it easily in the log file.
     my @err_host_names = keys %err_hosts;
 
     @err_host_names = sort { $err_hosts{$b} <=> $err_hosts{$a} } @err_host_names;
-    my @log_entries  = ("Failed hosts:");
-    my $total_failed = 0;
+    my @log_entries     = ("Failed hosts:");
+    my $num_jobs_failed = 0;
     for my $h (@err_host_names) {
-      $total_failed += $err_hosts{$h};
+      $num_jobs_failed += $err_hosts{$h};
       push @log_entries, sprintf( "   %3d %s", $err_hosts{$h}, $h );
     }
-    push @log_entries, sprintf( "TOTAL FAILED: %s of %s", $total_failed, $total_jobs );
 
-    $self->_report_log( '', @log_entries );
+    $self->_report_log( sprintf( "Jobs failed: %s of %s", $num_jobs_failed, $num_jobs ) );
+    $self->num_jobs_failed($num_jobs_failed);
+
+    $self->_report_log(@log_entries);
   } else {
     $self->_report_log('all nodes finished successfully');
   }
@@ -208,10 +243,10 @@ sub _report_crashed_job {
 
   $self->_report_cmd(
     "cd '$log_data->{cwd}' && $s->{job_cmd} --range $s->{range} --job_id $s->{job_id} --id $log_data->{id}");
-  $self->_report_log( "Node " . $log_data->{id} . " crashed" );
-  $self->_report_log("    log: $s->{log_file}");
-  $self->_report_log("    err: $s->{err_file}");
-  $self->_report_log("    out: $s->{out_file}");
+  $self->_report_node_log( "Node " . $log_data->{id} . " crashed" );
+  $self->_report_node_log("    log: $s->{log_file}");
+  $self->_report_node_log("    err: $s->{err_file}");
+  $self->_report_node_log("    out: $s->{out_file}");
 
   return;
 }
@@ -223,7 +258,7 @@ sub _report_missing_job {
   #replace job array numbers with worker id, to emulate the environment of the original array job
 
   $self->_report_cmd("cd '$s->{cwd}' && $s->{job_cmd} --job_id $s->{job_id} --id $s->{id}");
-  $self->_report_log( "Node " . $s->{id} . " crashed, NO_LOG NO_ERR NO_OUT" );
+  $self->_report_node_log( "Node " . $s->{id} . " crashed, NO_LOG NO_ERR NO_OUT" );
 
   return;
 }
@@ -238,10 +273,10 @@ sub _report_error_job {
 
     $self->_report_cmd(
       "cd '$log_data->{cwd}' && $s->{job_cmd} --range $task_id --job_id $s->{job_id} --id $log_data->{id}");
-    $self->_report_log( "Node " . $log_data->{id} . " had error(s)" );
-    $self->_report_log("    log: $s->{log_file}");
-    $self->_report_log("    err: $s->{err_file}");
-    $self->_report_log("    out: $s->{out_file}");
+    $self->_report_node_log( "Node " . $log_data->{id} . " had error(s)" );
+    $self->_report_node_log("    log: $s->{log_file}");
+    $self->_report_node_log("    err: $s->{err_file}");
+    $self->_report_node_log("    out: $s->{out_file}");
   }
   return;
 }
@@ -255,9 +290,9 @@ sub notify {
   return unless ( $c->{notify} );
 
   my %info = (
-    subject => '[LOG]' . $self->_gen_subject(),
-    message => join( "\n", @{ $self->_log_report } ) . "\n",
-    from    => $self->_get_from_address(),
+    subject => '[LOG]' . $self->subject(),
+    message => $self->gen_report,
+    from    => $self->from_address(),
   );
 
   if ( $c->{notify}{mail} ) {
@@ -274,43 +309,80 @@ sub notify {
       last unless ( $n->notify( \%info ) );
     }
   }
+  if ( $c->{notify}{script} ) {
+    my $bin = $c->{notify}{script};
+    if(-x $bin) {
+      open my $fh, '|-', $bin or die "Can't pipe to script >> $bin <<: $!";
+      say $fh jfreeze(\%info);
+      close $fh;
+    }
+  }
 
   return;
 }
 
-sub _gen_subject {
+sub subject {
   my $self = shift;
   my $c    = $self->c;
 
   return
       '['
     . localtime . ']['
-    . $c->{job_id} . '] '
+    . $c->{job_id} . ']['
+    . $self->log_status . '] '
     . $c->{job_name} . ' ('
-    . $self->_get_from_address() . ')';
+    . $self->from_address() . ')';
 }
 
-sub _get_from_address {
+sub log_status {
+  my ($self) = @_;
+
+  my $status;
+  if ( !$self->num_jobs || $self->num_jobs_failed == $self->num_jobs ) {
+    $status = 'ALL ERR';
+  } elsif ( $self->num_jobs_failed ) {
+    $status = $self->num_jobs_failed . '/' . $self->num_jobs . ' ERR';
+  } else {
+    $status = 'ALL OK';
+  }
+
+  return $status;
+}
+
+sub from_address {
   return $ENV{SGE_O_LOGNAME} && $ENV{SGE_O_HOST}
     ? join( '@', $ENV{SGE_O_LOGNAME}, $ENV{SGE_O_HOST} )
     : join( '@', $ENV{USER}, ( $ENV{HOSTNAME} || hostname() ) );
 }
 
+sub gen_report {
+  my ($self, $full) = @_;
+  my @node_report = @{ $self->_node_log };
+  @node_report = ( @node_report[ 0 .. 13 ], '...' ) if (@node_report > 17  && !$full);
+  return join( "\n", $self->subject(), '', @{ $self->_log }, '', @node_report ) . "\n";
+}
+
+sub restart_script {
+  my ($self) = @_;
+  return join( "\n", @{ $self->_cmd_script } ) . "\n";
+}
+
 sub write {
   my ($self) = @_;
 
-  my $cmd_f = $self->error_cmd_file;
+  my $cmd_f = $self->failed_restart_file;
 
   open my $cmd_fh, '>', $cmd_f or confess "Can't open filehandle: $!";
-  print $cmd_fh join "\n", @{ $self->_cmd_script };
+  print $cmd_fh $self->restart_script;
   $cmd_fh->close;
 
   chmod 0755, $cmd_f;
 
-  my $log_f = $self->error_log_file;
+  my $log_f = $self->failed_log_file;
 
   open my $log_fh, '>', $log_f or confess "Can't open filehandle: $!";
-  print $log_fh join( "\n", @{ $self->_log_report } ), "\n";
+
+  print $log_fh $self->gen_report(1);
   $log_fh->close;
 
   #CREATE SCRIPT TO UPDATE RERUN JOBS
@@ -330,7 +402,7 @@ sub _write_update_script {
   push @post_log_cmd, '-o', $c->{stdout_dir};
   push @post_log_cmd, @{ $c->{cmd} }, '--node_log', $c->{job_id}, $self->config_file;
 
-  my $update_log_file = catfile( $c->{tmp_dir}, "update.$c->{job_name}.j$c->{job_id}.sh" );
+  my $update_log_file = $self->failed_update_file;
 
   open my $update_log_fh, '>', $update_log_file or confess "Can't open filehandle: $!";
   print $update_log_fh join( " ", "cd", "'" . $c->{working_dir} . "'", '&&', @post_log_cmd ), "\n";
